import unittest

from ..domains import *
from ..linexprs import symbols
from ..polyhedra import *


class TestDomain(unittest.TestCase):

    def setUp(self):
        x, y = symbols('x y')
        self.square1 = Polyhedron(inequalities=[x, 2 - x, y, 2 - y])
        self.square2 = Polyhedron(inequalities=[x - 1, 3 - x , y - 1, 3 - y]) #correct representation
        self.square3 = Polyhedron(inequalities=[x, 3 - x, y, 3 - y])
        self.square4 = Polyhedron(inequalities=[x - 1, 2 - x, y - 1, 2 - y])
        self.square5 = Polyhedron(inequalities=[x - 3, 6 - x, y - 3, 6 -y])
        self.square6 = Polyhedron(equalities=[3 - y], inequalities=[x - 1, 3 - x, y - 1])
        self.unbound_poly = Polyhedron(inequalities=[x, 3 - x, y])
        self.universe = Polyhedron([])
        self.empty = Empty
        self.disjoint = And(Ge(x, 0), Ge(-x + 2, 0), Ge(y, 0), Ge(-y + 2, 0))
        self.complement = Or(Ge(-x - 1, 0), Ge(x - 3, 0), And(Ge(x, 0), Ge(-x + 2, 0), Ge(-y - 1, 0)), And(Ge(x, 0), Ge(-x + 2, 0), Ge(y - 3, 0)))
        self.hull = And(Ge(x, 0), Ge(-x + 2, 0), Ge(y, 0), Ge(-y + 2, 0))
        self.dropped = And(Ge(y, 0), Ge(-y + 2, 0))
        self.sample = And(Eq(y - 3, 0), Eq(x - 1, 0))
        self.intersection = And(Ge(x - 1, 0), Ge(-x + 2, 0), Ge(y - 1, 0), Ge(-y + 2, 0))
        self.union = Or(And(Ge(x, 0), Ge(-x + 2, 0), Ge(y, 0), Ge(-y + 2, 0)), And(Ge(x - 1, 0), Ge(-x + 3, 0), Ge(y - 1, 0), Ge(-y + 3, 0)))
        self.sum1 = Or(And(Ge(x, 0), Ge(-x + 2, 0), Ge(y, 0), Ge(-y + 2, 0)), And(Ge(x - 1, 0), Ge(-x + 3, 0), Ge(y - 1, 0), Ge(-y + 3, 0)))
        self.sum2 =And(Ge(x, 0), Ge(y, 0), Ge(-y + 3, 0), Ge(-x + 3, 0), Ge(x - y + 2, 0), Ge(-x + y + 2, 0))
        self.difference1 = Or(And(Eq(x - 3, 0), Ge(y - 1, 0), Ge(-y + 3, 0)), And(Eq(y - 3, 0), Ge(x - 1, 0), Ge(-x + 2, 0)))
        self.difference2 = And(Ge(x + y - 4, 0), Ge(-x + 3, 0), Ge(-y + 3, 0))
        self.lexmin = And(Eq(y, 0), Eq(x, 0))
        self.lexmax = And(Eq(y - 2, 0), Eq(x - 2, 0))

    def test_new(self):
        with self.assertRaises(TypeError):
            Polyhedron(1)

    def test_disjoint(self):
        self.assertEqual(self.square1.disjoint(), self.disjoint)

    def test_isempty(self):
        self.assertFalse(self.square1.isempty())
        self.assertTrue(self.empty.isempty())

    def test_isuniverse(self):
        self.assertFalse(self.square1.isuniverse())
        self.assertTrue(self.universe.isuniverse())

    def test_isbounded(self):
        self.assertTrue(self.square1.isbounded())
        self.assertFalse(self.unbound_poly.isbounded())

    def test_eq(self):
        self.assertTrue(self.square1 == self.square1)
        self.assertFalse(self.square1 == self.square2)

    def test_isdisjoint(self):
        self.assertFalse(self.square1.isdisjoint(self.square2))
        self.assertTrue(self.square1.isdisjoint(self.square5))

    def test_issubset(self):
        self.assertTrue(self.square4.issubset(self.unbound_poly))
        self.assertFalse(self.square1.issubset(self.square2))

    def test_le(self):
        self.assertTrue(self.square4 <= self.square3)
        self.assertFalse(self.square3 <= self.square4)

    def test_lt(self):
        self.assertTrue(self.square4 < self.square3)
        self.assertFalse(self.square3 < self.square4)

    def test_complement(self):
        self.assertEqual(~self.square1, self.complement)

    def test_polyhedral_hull(self):
        self.assertEqual(self.square1.polyhedral_hull(), self.hull)

    def test_project_out(self):
        self.assertEqual(self.square1.project_out('x'), self.dropped)
        self.assertEqual(self.square1.project_out('x y'), self.universe)
        self.assertEqual(self.universe.project_out(' '), self.universe)
        self.assertEqual(self.empty.project_out(' '), Empty)

    def test_simplify(self):
        self.assertEqual(self.universe.simplify(), self.universe)
        self.assertEqual(self.empty.simplify(), Empty)

    def test_sample(self):
        self.assertEqual(self.empty.sample(), Empty)
        self.assertEqual(self.universe.sample(), self.universe)
        self.assertEqual(self.square6.sample(), self.sample)

    def test_intersection(self):
        self.assertEqual(self.square1.intersection(self.square2), self.intersection)

    def test_and(self):
        self.assertEqual(self.square2 & self.square1, self.intersection)

    def test_union(self):
        self.assertEqual(self.square1.union(self.square2), self.union)

    def test_or(self):
        self.assertEqual(self.square1 | self.square2, self.union)

    def test_add(self):
        self.assertEqual(self.square2 + self.square1, self.sum1)
        self.assertEqual(Polyhedron(self.square1 + self.square2), self.sum2)

    def test_difference(self):
        self.assertEqual(self.square2 - self.square1, self.difference1)
        self.assertEqual(Polyhedron(self.square2 - self.square1), self.difference2)
        self.assertEqual(self.square2 - self.square2, Empty)
        self.assertEqual(self.universe - self.universe, Empty)

    def test_lexmin(self):
        self.assertEqual(self.square1.lexmin(), self.lexmin)

    def test_lexmax(self):
        self.assertEqual(self.square1.lexmax(), self.lexmax)
